@@ -1,19 +1,29 @@
+// SPDX-FileCopyrightText: 2011 Vamsi Krishna Brahmajosyula <vamsikrishna.brahmajosyula@gmail.com>
+// SPDX-FileCopyrightText: 2011 Giovanni Campagna <gcampagna@src.gnome.org>
+// SPDX-FileCopyrightText: 2013 Debarshi Ray <debarshir@gnome.org>
+// SPDX-FileCopyrightText: 2013 Florian Müllner <fmuellner@gnome.org>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
-/* exported init enable disable */
+import Atk from 'gi://Atk';
+import Clutter from 'gi://Clutter';
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import GMenu from 'gi://GMenu';
+import GObject from 'gi://GObject';
+import Gtk from 'gi://Gtk';
+import Meta from 'gi://Meta';
+import Shell from 'gi://Shell';
+import St from 'gi://St';
+import {EventEmitter} from 'resource:///org/gnome/shell/misc/signals.js';
 
-const {
-    Atk, Clutter, Gio, GLib, GMenu, GObject, Gtk, Meta, Shell, St,
-} = imports.gi;
-const Signals = imports.signals;
+import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
 
-const DND = imports.ui.dnd;
-const ExtensionUtils = imports.misc.extensionUtils;
-const Main = imports.ui.main;
-const PanelMenu = imports.ui.panelMenu;
-const PopupMenu = imports.ui.popupMenu;
-
-const Gettext = imports.gettext.domain('gnome-shell-extensions');
-const _ = Gettext.gettext;
+import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
+import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
+import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
 
 const appSys = Shell.AppSystem.get_default();
 
@@ -25,10 +35,13 @@ const NAVIGATION_REGION_OVERSHOOT = 50;
 Gio._promisify(Gio._LocalFilePrototype, 'query_info_async', 'query_info_finish');
 Gio._promisify(Gio._LocalFilePrototype, 'set_attributes_async', 'set_attributes_finish');
 
-var ApplicationMenuItem = GObject.registerClass(
 class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
-    _init(button, app) {
-        super._init();
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(button, app) {
+        super();
         this._app = app;
         this._button = button;
 
@@ -44,11 +57,8 @@ class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
         this.label_actor = appLabel;
 
         let textureCache = St.TextureCache.get_default();
-        let iconThemeChangedId = textureCache.connect('icon-theme-changed',
-            this._updateIcon.bind(this));
-        this.connect('destroy', () => {
-            textureCache.disconnect(iconThemeChangedId);
-        });
+        textureCache.connectObject('icon-theme-changed',
+            () => this._updateIcon(), this);
         this._updateIcon();
 
         this._delegate = this;
@@ -94,12 +104,15 @@ class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
         icon.style_class = 'icon-dropshadow';
         this._iconBin.set_child(icon);
     }
-});
+}
 
-var CategoryMenuItem = GObject.registerClass(
 class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
-    _init(button, category) {
-        super._init();
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(button, category) {
+        super();
         this._category = category;
         this._button = button;
 
@@ -112,7 +125,7 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         else
             name = _('Favorites');
 
-        this.add_child(new St.Label({ text: name }));
+        this.add_child(new St.Label({text: name}));
         this.connect('motion-event', this._onMotionEvent.bind(this));
         this.connect('notify::active', this._onActiveChanged.bind(this));
     }
@@ -181,11 +194,14 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
     }
 
     _onMotionEvent(actor, event) {
-        let device = event.get_device();
-        if (!device.get_grabbed_actor()) {
+        if (!this._grab) {
             this._oldX = -1;
             this._oldY = -1;
-            device.grab(this);
+            const grab = global.stage.grab(this);
+            if (grab.get_seat_state() !== Clutter.GrabState.NONE)
+                this._grab = grab;
+            else
+                grab.dismiss();
         }
         this.hover = true;
 
@@ -195,11 +211,12 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._oldX = -1;
         this._oldY = -1;
         this.hover = false;
-        device.ungrab();
+        this._grab?.dismiss();
+        delete this._grab;
 
-        let source = event.get_source();
-        if (source instanceof St.Widget)
-            source.sync_hover();
+        const targetActor = global.stage.get_event_actor(event);
+        if (targetActor instanceof St.Widget)
+            targetActor.sync_hover();
 
         return false;
     }
@@ -211,7 +228,7 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._button.selectCategory(this._category);
         this._button.scrollToCatButton(this);
     }
-});
+}
 
 class ApplicationsMenu extends PopupMenu.PopupMenu {
     constructor(sourceActor, arrowAlignment, arrowSide, button) {
@@ -230,8 +247,10 @@ class ApplicationsMenu extends PopupMenu.PopupMenu {
     }
 }
 
-class DesktopTarget {
+class DesktopTarget extends EventEmitter {
     constructor() {
+        super();
+
         this._desktop = null;
         this._desktopDestroyedId = 0;
 
@@ -258,9 +277,7 @@ class DesktopTarget {
 
     _setDesktop(desktop) {
         if (this._desktop) {
-            this._desktop.disconnect(this._desktopDestroyedId);
-            this._desktopDestroyedId = 0;
-
+            this._desktop.disconnectObject(this);
             delete this._desktop._delegate;
         }
 
@@ -268,9 +285,9 @@ class DesktopTarget {
         this.emit('desktop-changed');
 
         if (this._desktop) {
-            this._desktopDestroyedId = this._desktop.connect('destroy', () => {
+            this._desktop.connectObject('destroy', () => {
                 this._setDesktop(null);
-            });
+            }, this);
             this._desktop._delegate = this;
         }
     }
@@ -310,10 +327,7 @@ class DesktopTarget {
     }
 
     destroy() {
-        if (this._windowAddedId)
-            global.window_group.disconnect(this._windowAddedId);
-        this._windowAddedId = 0;
-
+        global.window_group.disconnectObject(this);
         this._setDesktop(null);
     }
 
@@ -348,12 +362,28 @@ class DesktopTarget {
         return true;
     }
 }
-Signals.addSignalMethods(DesktopTarget.prototype);
 
-let ApplicationsButton = GObject.registerClass(
+class MainLayout extends Clutter.BoxLayout {
+    static {
+        GObject.registerClass(this);
+    }
+
+    vfunc_get_preferred_height(container, forWidth) {
+        const [mainChild] = container;
+        const [minHeight, natHeight] =
+            mainChild.get_preferred_height(forWidth);
+
+        return [minHeight, natHeight + MENU_HEIGHT_OFFSET];
+    }
+}
+
 class ApplicationsButton extends PanelMenu.Button {
-    _init() {
-        super._init(1.0, null, false);
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor() {
+        super(1.0, null, false);
 
         this.setMenu(new ApplicationsMenu(this, 1.0, St.Side.TOP, this));
         Main.panel.menuManager.addMenu(this.menu);
@@ -363,29 +393,27 @@ class ApplicationsButton extends PanelMenu.Button {
         // role ATK_ROLE_MENU like other elements of the panel.
         this.accessible_role = Atk.Role.LABEL;
 
-        let hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
-
         this._label = new St.Label({
             text: _('Applications'),
             y_expand: true,
             y_align: Clutter.ActorAlign.CENTER,
         });
-        hbox.add_child(this._label);
-        hbox.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
 
-        this.add_actor(hbox);
+        this.add_actor(this._label);
         this.name = 'panelApplications';
         this.label_actor = this._label;
 
-        this._showingId = Main.overview.connect('showing', () => {
-            this.add_accessible_state(Atk.StateType.CHECKED);
-        });
-        this._hidingId = Main.overview.connect('hiding', () => {
-            this.remove_accessible_state(Atk.StateType.CHECKED);
-        });
-        Main.layoutManager.connect('startup-complete',
-            this._setKeybinding.bind(this));
-        this._setKeybinding();
+        Main.overview.connectObject(
+            'showing', () => this.add_accessible_state(Atk.StateType.CHECKED),
+            'hiding', () => this.remove_accessible_state(Atk.StateType.CHECKED),
+            this);
+
+        Main.wm.addKeybinding(
+            'apps-menu-toggle-menu',
+            Extension.lookupByURL(import.meta.url).getSettings(),
+            Meta.KeyBindingFlags.IGNORE_AUTOREPEAT,
+            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
+            () => this.menu.toggle());
 
         this._desktopTarget = new DesktopTarget();
         this._desktopTarget.connect('app-dropped', () => {
@@ -397,16 +425,16 @@ class ApplicationsButton extends PanelMenu.Button {
             });
         });
 
-        this._tree = new GMenu.Tree({ menu_basename: 'applications.menu' });
-        this._treeChangedId = this._tree.connect('changed',
-            this._onTreeChanged.bind(this));
+        this._tree = new GMenu.Tree({menu_basename: 'applications.menu'});
+        this._tree.connectObject('changed',
+            () => this._onTreeChanged(), this);
 
         this._applicationsButtons = new Map();
         this.reloadFlag = false;
         this._createLayout();
         this._display();
-        this._installedChangedId = appSys.connect('installed-changed',
-            this._onTreeChanged.bind(this));
+        appSys.connectObject('installed-changed',
+            () => this._onTreeChanged(), this);
     }
 
     _onTreeChanged() {
@@ -418,29 +446,12 @@ class ApplicationsButton extends PanelMenu.Button {
         }
     }
 
-    _createVertSeparator() {
-        let separator = new St.DrawingArea({
-            style_class: 'calendar-vertical-separator',
-            pseudo_class: 'highlighted',
-        });
-        separator.connect('repaint', this._onVertSepRepaint.bind(this));
-        return separator;
-    }
-
     _onDestroy() {
         super._onDestroy();
 
-        Main.overview.disconnect(this._showingId);
-        Main.overview.disconnect(this._hidingId);
-        appSys.disconnect(this._installedChangedId);
-        this._tree.disconnect(this._treeChangedId);
-        this._tree = null;
+        delete this._tree;
 
-        Main.wm.setCustomKeybindingHandler('panel-main-menu',
-            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
-            Main.sessionMode.hasOverview
-                ? Main.overview.toggle.bind(Main.overview)
-                : null);
+        Main.wm.removeKeybinding('apps-menu-toggle-menu');
 
         this._desktopTarget.destroy();
     }
@@ -456,21 +467,6 @@ class ApplicationsButton extends PanelMenu.Button {
         return super._onMenuKeyPress(actor, event);
     }
 
-    _onVertSepRepaint(area) {
-        let cr = area.get_context();
-        let themeNode = area.get_theme_node();
-        let [width, height] = area.get_surface_size();
-        let stippleColor = themeNode.get_color('-stipple-color');
-        let stippleWidth = themeNode.get_length('-stipple-width');
-        let x = Math.floor(width / 2) + 0.5;
-        cr.moveTo(x, 0);
-        cr.lineTo(x, height);
-        Clutter.cairo_set_source_color(cr, stippleColor);
-        cr.setDash([1, 3], 1); // Hard-code for now
-        cr.setLineWidth(stippleWidth);
-        cr.stroke();
-    }
-
     _onOpenStateChanged(menu, open) {
         if (open) {
             if (this.reloadFlag) {
@@ -482,12 +478,6 @@ class ApplicationsButton extends PanelMenu.Button {
         super._onOpenStateChanged(menu, open);
     }
 
-    _setKeybinding() {
-        Main.wm.setCustomKeybindingHandler('panel-main-menu',
-            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
-            () => this.menu.toggle());
-    }
-
     _redisplay() {
         this.applicationsBox.destroy_all_children();
         this.categoriesBox.destroy_all_children();
@@ -508,7 +498,7 @@ class ApplicationsButton extends PanelMenu.Button {
                 }
                 let app = appSys.lookup_app(id);
                 if (!app)
-                    app = new Shell.App({ app_info: entry.get_app_info() });
+                    app = new Shell.App({app_info: entry.get_app_info()});
                 if (app.get_app_info().should_show())
                     this.applicationsByCategory[categoryId].push(app);
             } else if (nextType === GMenu.TreeItemType.SEPARATOR) {
@@ -554,13 +544,11 @@ class ApplicationsButton extends PanelMenu.Button {
     _createLayout() {
         let section = new PopupMenu.PopupMenuSection();
         this.menu.addMenuItem(section);
-        this.mainBox = new St.BoxLayout({ vertical: false });
-        this.leftBox = new St.BoxLayout({ vertical: true });
+        this.mainBox = new St.BoxLayout({layoutManager: new MainLayout()});
+        this.leftBox = new St.BoxLayout({vertical: true});
         this.applicationsScrollBox = new St.ScrollView({
-            x_fill: true,
-            y_fill: false,
-            y_align: St.Align.START,
             style_class: 'apps-menu vfade',
+            x_expand: true,
         });
         this.applicationsScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
         let vscroll = this.applicationsScrollBox.get_vscroll_bar();
@@ -571,44 +559,26 @@ class ApplicationsButton extends PanelMenu.Button {
             this.menu.passEvents = false;
         });
         this.categoriesScrollBox = new St.ScrollView({
-            x_fill: true,
-            y_fill: false,
-            y_align: St.Align.START,
             style_class: 'vfade',
         });
         this.categoriesScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
         vscroll = this.categoriesScrollBox.get_vscroll_bar();
         vscroll.connect('scroll-start', () => (this.menu.passEvents = true));
         vscroll.connect('scroll-stop', () => (this.menu.passEvents = false));
-        this.leftBox.add(this.categoriesScrollBox, {
-            expand: true,
-            x_fill: true,
-            y_fill: true,
-            y_align: St.Align.START,
-        });
+        this.leftBox.add_child(this.categoriesScrollBox);
 
-        this.applicationsBox = new St.BoxLayout({ vertical: true });
+        this.applicationsBox = new St.BoxLayout({vertical: true});
         this.applicationsScrollBox.add_actor(this.applicationsBox);
-        this.categoriesBox = new St.BoxLayout({ vertical: true });
+        this.categoriesBox = new St.BoxLayout({vertical: true});
         this.categoriesScrollBox.add_actor(this.categoriesBox);
 
         this.mainBox.add(this.leftBox);
-        this.mainBox.add(this._createVertSeparator(), {
-            expand: false,
-            x_fill: false,
-            y_fill: true,
-        });
-        this.mainBox.add(this.applicationsScrollBox, {
-            expand: true,
-            x_fill: true,
-            y_fill: true,
-        });
+        this.mainBox.add_child(this.applicationsScrollBox);
         section.actor.add_actor(this.mainBox);
     }
 
     _display() {
         this._applicationsButtons.clear();
-        this.mainBox.style = 'width: 35em;';
         this.mainBox.hide();
 
         // Load categories
@@ -638,12 +608,6 @@ class ApplicationsButton extends PanelMenu.Button {
 
         // Load applications
         this._displayButtons(this._listApplications(null));
-
-        let themeContext = St.ThemeContext.get_for_stage(global.stage);
-        let scaleFactor = themeContext.scale_factor;
-        let categoriesHeight = this.categoriesBox.height / scaleFactor;
-        let height = Math.round(categoriesHeight) + MENU_HEIGHT_OFFSET;
-        this.mainBox.style += `height: ${height}px`;
     }
 
     selectCategory(dir) {
@@ -691,21 +655,19 @@ class ApplicationsButton extends PanelMenu.Button {
 
         return applist;
     }
-});
-
-let appsMenuButton;
-
-function enable() {
-    appsMenuButton = new ApplicationsButton();
-    let index = Main.sessionMode.panel.left.indexOf('activities') + 1;
-    Main.panel.addToStatusArea('apps-menu', appsMenuButton, index, 'left');
 }
 
-function disable() {
-    Main.panel.menuManager.removeMenu(appsMenuButton.menu);
-    appsMenuButton.destroy();
-}
+export default class AppsMenuExtension extends Extension {
+    enable() {
+        this._appsMenuButton = new ApplicationsButton();
+        const index = Main.sessionMode.panel.left.indexOf('activities') + 1;
+        Main.panel.addToStatusArea(
+            'apps-menu', this._appsMenuButton, index, 'left');
+    }
 
-function init() {
-    ExtensionUtils.initTranslations();
+    disable() {
+        Main.panel.menuManager.removeMenu(this._appsMenuButton.menu);
+        this._appsMenuButton.destroy();
+        delete this._appsMenuButton;
+    }
 }
